@@ -11,17 +11,18 @@ import shutil
 # Pour Exécution de programmes
 import subprocess
 
-from colored import fg, bg, attr
+from rich.console import Console
+from rich.panel import Panel
+from rich.progress import Progress
+
+#from rich import pretty
+from rich import print
 # Helpers
 
 def search_files(name, d='.'):
     return [os.path.join(root, f) for root, _, files in os.walk(d) for f in files if f == name]
-def choice_str(choices, target=''):
-    res = '. ' + str(target) + '\n' + '│\n'
-    for choice in choices[:-1]:
-      res = res + '├── ' + str(choice) + '\n'
-    res = res + '└── ' + choices[-1]
-    return res
+
+#pretty.install()
 
 class FastEval:
     """
@@ -33,44 +34,35 @@ class FastEval:
     """
     def __init__(self, args):
         "docstring"
-        self.ecolor = bg('indian_red_1a') + fg('white')
-        #self.ecolor = fg('red_3a')
-        #self.wcolor = bg('orange_1') + fg('white')
-        self.wcolor = fg('orange_1')
-        #self.icolor = bg('deep_sky_blue_2') + fg('white')
-        #self.icolor = fg('medium_turquoise') + attr('bold')
-        self.icolor = fg('light_sea_green') + attr('bold')
-        self.rcolor = attr('reset')
+        self.console = Console()
         if args.workspace:
             self.workspace_path = os.path.abspath(os.path.expanduser(args.workspace))
         else:
             self.workspace_path = os.path.join(os.getcwd(), 'submissions')
-        print(f'Using  {self.info_str(self.workspace_path)} as workspace. {self.info_str("✓")}')
+        print(f'Using  {self.workspace_path} as workspace. ✓')
 
         self.archive_path = os.path.expanduser(args.archive_path)
         if not os.path.exists(self.archive_path):
-            print('Given  {}'
-                  ' does not exist, exiting...'.format(self.erro_str(self.archive_path)),
-                  file=sys.stderr)
+            print(f'Given  {self.archive_path}'
+                  ' does not exist, exiting...', file=sys.stderr)
             sys.exit()
 
         self.verbosity = args.verbosity
         config_path = os.path.expanduser(args.config)
-        assert os.path.isfile(config_path), "{} is not a file.".format(self.erro_str(config_path))
+        assert os.path.isfile(config_path), f'{config_path} is not a file.'
 
         with open(config_path, 'r') as fp:
             config = json.load(fp)
-        print(f'Loaded {self.info_str(config_path)} savefile. {self.info_str("✓")}')
+        print(f'Loaded {config_path} config file. ✓')
         self.required_files = config['required_files']
 
         if len(config['reference_folder']) > 0:
             self.ref_path = os.path.expanduser(config['reference_folder'])
             if not os.path.isdir(self.ref_path):
-                print('Given  {}'
-                  ' does not exist, exiting...'.format(self.erro_str(self.ref_path)),
-                  file=sys.stderr)
+                print(f'Given  {self.ref_path}'
+                  ' does not exist, exiting...', file=sys.stderr)
                 sys.exit()
-            print(f'Using  {self.info_str(self.ref_path)} as reference folder. {self.info_str("✓")}')
+            print(f'Using  {self.ref_path} as reference folder. ✓')
         else:
             self.ref_path = None
             print('Not using ref folder')
@@ -99,7 +91,7 @@ class FastEval:
         if self.pass_count == 0:
             shutil.unpack_archive(self.archive_path, self.workspace_path)
             submissions = self.clean_dirs()
-            print('Processing {} projects...\n'.format(len(submissions)))
+            print(f'Processing {len(submissions)} projects...\n')
             self.submissions = {key: dict(value, **{'step' : '0_prep', 'steps': {'0_prep' : {},
                                                                                  '1_comp' : {},
                                                                                  '2_exec' : {},
@@ -110,7 +102,7 @@ class FastEval:
             self.prep_step()
             self.gen_csv()
         else:
-            print('Processing {} projects...\n'.format(len(self.submissions)))
+            print(f'Processing {len(self.submissions)} projects...\n')
             self.check_prep()
 
         self.print_step_errors('0_prep')
@@ -134,9 +126,9 @@ class FastEval:
     
             self.pass_count = data['pass_count'] + 1
             self.submissions = data['submissions']
-            print(f'Loaded {self.info_str(data_file)} savefile. {self.info_str("✓")}\n')
+            print(f'Loaded {data_file} savefile. ✓\n')
         except FileNotFoundError:
-            print(f'Using  {self.info_str(data_file)} savefile. {self.info_str("✓")}\n')
+            print(f'Using  {data_file} savefile. ✓\n')
             self.pass_count = 0
     def write_data(self):
         data_file = os.path.join(self.workspace_path, 'data.json')
@@ -145,7 +137,7 @@ class FastEval:
                 json.dump({'pass_count': self.pass_count,
                            'submissions': self.submissions},
                           fp, sort_keys=True, indent=4, ensure_ascii=False)
-            print(f'Wrote  {self.info_str(data_file)} savefile. {self.info_str("✓")}')
+            print(f'Wrote  {data_file} savefile. ✓')
         except:
             print('Error while writing : \n => {}\n'.format(data_file),
                   file=sys.stderr)
@@ -165,15 +157,13 @@ class FastEval:
             os.mkdir(raw_dir)
             for o in os.listdir(self.submissions[sub]['path']):
                 shutil.move(os.path.join(self.submissions[sub]['path'],o), raw_dir)
-            #files = [os.path.join(raw_dir, o) for o in os.listdir(raw_dir)]
             files = [os.path.join(raw_dir, f) for root, _, files in os.walk(raw_dir) for f in files]
-            print(files)
             for f in files:
                 try:
                     shutil.unpack_archive(f, raw_dir)
                     #os.remove(f)
                 except shutil.ReadError:
-                    print('Unpack ' + self.warn_str(f) + ' failed.')
+                    print(f'Unpack {f} failed.')
     
     def copy_ref(self):
         if self.ref_path is not None:
@@ -182,109 +172,128 @@ class FastEval:
     
     def prep_step(self):
         to_prep = [sub for sub in self.submissions if self.submissions[sub]['step'] == '0_prep']
-        print('Preparing  {} projects...'.format(len(to_prep)))
-        for sub in to_prep:
-            raw_dir = os.path.join(self.submissions[sub]['path'], 'raw')
-            eval_dir = os.path.join(self.submissions[sub]['path'], 'eval')
+        print(f'Preparing  {len(to_prep)} projects...')
+        with Progress(transient=True) as progress:
+            task = progress.add_task(f'Preparing...', total=len(to_prep))
+            for sub in to_prep:
+                raw_dir = os.path.join(self.submissions[sub]['path'], 'raw')
+                eval_dir = os.path.join(self.submissions[sub]['path'], 'eval')
     
-            if not os.path.exists(eval_dir):
-                os.mkdir(eval_dir)
+                if not os.path.exists(eval_dir):
+                    os.mkdir(eval_dir)
     
-            missing_files = []
+                missing_files = []
     
-            # Search every required files one by one
-            for f in self.required_files:
-                # List cadidates for searched file
-                student_code = search_files(f, raw_dir)
-                # Filter files in a "__MACOS" directory
-                student_code = [s for s in student_code if '__MACOS' not in s]
-                if len(student_code) == 1:
-                    shutil.copyfile(student_code[0], os.path.join(eval_dir, f))
-                elif len(student_code) == 0:
-                    missing_files.append(f)
+                # Search every required files one by one
+                for f in self.required_files:
+                    # List cadidates for searched file
+                    student_code = search_files(f, raw_dir)
+                    # Filter files in a "__MACOS" directory
+                    student_code = [s for s in student_code if '__MACOS' not in s]
+                    if len(student_code) == 1:
+                        shutil.copyfile(student_code[0], os.path.join(eval_dir, f))
+                    elif len(student_code) == 0:
+                        missing_files.append(f)
+                    else:
+                        msg = 'You need to manually copy one of those files'
+                        msg = msg + choice_str(student_code, f)
+                        self.submissions[sub]['steps']['0_prep']['msg'] = msg
+    
+                # Update missing files if needed
+                if missing_files:
+                    if 'missing_files' not in self.submissions[sub]['steps']['0_prep']:
+                        self.submissions[sub]['steps']['0_prep']['missing_files'] = missing_files
+                    else:
+                        self.submissions[sub]['steps']['0_prep']['missing_files'].extend(missing_files)
                 else:
-                    msg = 'You need to manually copy one of those files'
-                    msg = msg + choice_str(student_code, f)
-                    self.submissions[sub]['steps']['0_prep']['msg'] = msg
-    
-            # Update missing files if needed
-            if missing_files:
-                if 'missing_files' not in self.submissions[sub]['steps']['0_prep']:
-                    self.submissions[sub]['steps']['0_prep']['missing_files'] = missing_files
-                else:
-                    self.submissions[sub]['steps']['0_prep']['missing_files'].extend(missing_files)
-            else:
-                self.submissions[sub]['step'] = '1_comp'
+                    self.submissions[sub]['step'] = '1_comp'
+                progress.update(task, advance=1)
     
         to_prep = [sub for sub in self.submissions if self.submissions[sub]['step'] == '0_prep']
         if len(to_prep) == 0:
-            print(f'           0 fails. {self.info_str("✓")}')
+            print(f' 0 fails. ✓')
         else:
-            print('           ' + self.erro_str('{} fails.'.format(len(to_prep))) + '\n')
+            print(f' {len(to_prep)} fails.\n')
     def check_prep(self):
         to_check = [sub for sub in self.submissions if self.submissions[sub]['step'] == '0_prep']
-        print('Checking   {} projects...'.format(len(to_check)))
-        for sub in to_check:
-            eval_dir = os.path.join(self.submissions[sub]['path'], 'eval')
-            eval_files = [f for root, dirs, files in os.walk(eval_dir) for f in files]
+        print(f'Checking   {len(to_check)} projects...')
+        with Progress(transient=True) as progress:
+            task = progress.add_task(f'Checking...', total=len(to_check))
+            for sub in to_check:
+                eval_dir = os.path.join(self.submissions[sub]['path'], 'eval')
+                eval_files = [f for root, dirs, files in os.walk(eval_dir) for f in files]
     
     
-            missing_files = [f for f in self.required_files if f not in eval_files]
-            # Update missing files if needed
-            if missing_files:
-                self.submissions[sub]['steps']['0_prep']['missing_files'] = missing_files
-            else:
-                self.submissions[sub]['step'] = '1_comp'
+                missing_files = [f for f in self.required_files if f not in eval_files]
+                # Update missing files if needed
+                if missing_files:
+                    self.submissions[sub]['steps']['0_prep']['missing_files'] = missing_files
+                else:
+                    self.submissions[sub]['step'] = '1_comp'
     
-        to_check = [sub for sub in self.submissions if self.submissions[sub]['step'] == '0_prep']
+                to_check = [sub for sub in self.submissions if self.submissions[sub]['step'] == '0_prep']
+                progress.update(task, advance=1)
         if len(to_check) == 0:
-            print(f'           0 fails. {self.info_str("✓")}')
+            print(f' 0 fails. ✓')
         else:
-            print('           ' + self.erro_str('{} fails.'.format(len(to_check))) + '\n')
+            print(f' {len(to_check)} fails.\n')
+    
+    def format_output(self, out, max_lines=40):
+        if len(out) > max_lines:
+            return out[:max_lines//2] + ['<'] + ['truncated by fast-eval'] + ['>'] + out[-max_lines//2:]
+        return out
+    
     def exte_step(self, cmd, step='1_comp', label='Compiling', timeout=10):
         to_exec = [sub for sub in self.submissions if self.submissions[sub]['step'] == step]
-        print('{}  {} projects...'.format(label, len(to_exec)))
+        print(f'{label}  {len(to_exec)} projects...')
+        if not cmd:
+            print('Nothing to do.')
+            return 0
         root_dir = os.getcwd()
-        for sub in to_exec:
-            os.chdir(os.path.join(self.submissions[sub]['path'], 'eval'))
-            comp_ok = True
-            timeout_raised = False
-            for c in cmd:
-                try:
-                    completed_process = subprocess.run([c], capture_output=True, text=True, shell=True, timeout=timeout)
-                    if completed_process.returncode != 0:
-                        comp_ok=False
-                    cond = [len(completed_process.stderr) > 0, len(completed_process.stdout)]
-                    if any(cond) and c not in self.submissions[sub]['steps'][step]:
-                        self.submissions[sub]['steps'][step][c] = {}
-                    if cond[0]:
-                        self.submissions[sub]['steps'][step][c]['stderr'] = completed_process.stderr.split('\n')
-                    if cond[1]:
-                        out = completed_process.stdout.split('\n')
-                        if len(out) > 20:
-                            out = out[:10] + ['.'] + ['truncated by fast-eval'] + ['.'] + out[-10:]
-                        self.submissions[sub]['steps'][step][c]['stdout'] = out
-                except Exception as e:
-                    comp_ok=False
-                    if type(e) is subprocess.TimeoutExpired:
-                        self.submissions[sub]['steps'][step][c] = 'timeout'
+        with Progress(transient=True) as progress:
+            task = progress.add_task(f"[bold]{label}...", total=len(to_exec))
+            for sub in to_exec:
+                os.chdir(os.path.join(self.submissions[sub]['path'], 'eval'))
+                comp_ok = True
+                timeout_raised = False
+                for c in cmd:
+                    try:
+                        completed_process = subprocess.run([c], capture_output=True, text=True, shell=True, timeout=timeout)
+                        if completed_process.returncode != 0:
+                            comp_ok=False
+                            cond = [len(completed_process.stderr) > 0, len(completed_process.stdout)]
+                        if any(cond) and c not in self.submissions[sub]['steps'][step]:
+                            self.submissions[sub]['steps'][step][c] = {}
+                        if cond[0]:
+                            self.submissions[sub]['steps'][step][c]['stderr'] = self.format_output(
+                                completed_process.stderr.split('\n'))
+                        if cond[1]:
+                            out = completed_process.stdout.split('\n')
+                            self.submissions[sub]['steps'][step][c]['stdout'] = self.format_output(
+                                completed_process.stdout.split('\n'))
     
-            if comp_ok:
-                self.submissions[sub]['step'] = self.next_step(step)
+                    except Exception as e:
+                        comp_ok=False
+                        if type(e) is subprocess.TimeoutExpired:
+                            self.submissions[sub]['steps'][step][c] = 'timeout'
+    
+                if comp_ok:
+                    self.submissions[sub]['step'] = self.next_step(step)
+                progress.update(task, advance=1)
         os.chdir(root_dir)
         to_exec = [sub for sub in self.submissions if self.submissions[sub]['step'] == step]
         if len(to_exec) == 0:
-            print(f'           0 fails. {self.info_str("✓")}')
+            print(f' 0 fails. ✓')
         else:
-            print('           ' + self.erro_str('{} fails.'.format(len(to_exec))) + '\n')
+            print(f' {len(to_exec)} fails.\n')
     
     def cleanup(self):
         for c in self.cleanup_cmd:
             completed_process = subprocess.run(shlex.split(c))
             if completed_process.returncode == 0:
-                print(f'Cleanup : {c} {self.info_str("✓")}')
+                print(f'Cleanup : {c} ✓')
             else:
-                print(f'Cleanup : {c} {self.erro_str("❌")}')
+                print(f'Cleanup : {c} ❌')
     def export(self):
         outpath = os.path.join(self.workspace_path, 'readme.org')
         with open(outpath, 'w') as f:
@@ -305,7 +314,7 @@ class FastEval:
                 if steps['1_comp']:
                     usefull = False
                     for v in steps['1_comp'].values():
-                        if 'stderr' in v:
+                        if 'stderr' in v and v['stderr'] and len(v['stderr'][0])>0:
                             usefull = True
                     if usefull:
                         f.write(f'*** Erreurs de compilation\n')
@@ -343,14 +352,16 @@ class FastEval:
                     for k, v in steps['2_exec'].items():
                         f.write(f'#+begin_src bash\n')
                         f.write(f'{k}\n')
-                    f.write('#+end_src\n')
+                        f.write('#+end_src\n')
                     if 'stderr' in v:
+                        f.write('\nstderror\n')
                         f.write('\n#+name: stderror\n')
                         f.write(f'#+begin_example\n')
                         for line in v['stderr']:
                             f.write(f'{line}\n')
                         f.write('#+end_example\n')
                     if 'stdout' in v:
+                        f.write('\nstdout\n')
                         f.write('\n#+name: stdout\n')
                         f.write(f'#+begin_example\n')
                         for line in v['stdout']:
@@ -364,7 +375,7 @@ class FastEval:
         cmd = shlex.split(f'pandoc -s {inpath} -o {outpath} --highlight-style {style} --template=easy_template.html --standalone --toc')
         completed_process = subprocess.run(cmd)
         if completed_process.returncode == 0:
-            print(f'Wrote  {self.info_str(outpath)} readable file. {self.info_str("✓")}')
+            print(f'Wrote  {outpath} readable file. ✓')
         else:
            print('Error while generating html')
     
@@ -373,7 +384,6 @@ class FastEval:
         with open(outpath, 'w') as f:
             names = [s for s in self.submissions]
             names.sort()
-            print(names)
             for n in names:
                 f.write(f'{n}, note\n')
     def next_step(self, step):
@@ -385,17 +395,19 @@ class FastEval:
             return '3_eval'
         else:
             return 'done'
-    def erro_str(self, msg):
-        return self.ecolor + str(msg) + self.rcolor
-    def warn_str(self, msg):
-        return self.wcolor + str(msg) + self.rcolor
-    def info_str(self, msg):
-        return self.icolor + str(msg) + self.rcolor
     def print_step_errors(self, step):
         to_print = [sub for sub in self.submissions if self.submissions[sub]['step'] == step]
         if self.verbosity >= 1 and len(to_print) > 0:
             print(f"Fail list : {to_print}\n")
         if self.verbosity > 1:
             for s in to_print:
-                print(f'{s}\'s errors : \n {self.submissions[s]["steps"][step]}')
+                msg = f'{s}\'s errors : \n {self.submissions[s]["steps"][step]}'
+                #self.console.print(f'{s}\'s errors :', self.submissions[s]["steps"][step])
+                #self.console.print(msg)
+                #from rich import Pretty
+                #self.console.rule(f'{s}\'s errors :')
+                #self.console.print(self.submissions[s]['steps'][step])
+                self.console.print(Panel.fit(str(self.submissions[s]['steps'][step]), title=f'[red]{s}\'s errors :'))
+                #if len(self.submissions[s]["steps"][step]) > 0 and len(msg) < 1000:
+                #    print(msg)
         print("\n")
